@@ -25,14 +25,24 @@
 
 #ifdef USE_HAL
 
+#include <fstab.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <fstab.h>
+#include <sys/stat.h>
 #include <sys/sysmacros.h>
+#include <sys/types.h>
+#include <unistd.h>
+
+#include <glib.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-i18n.h>
 
+#include <libhal.h>
+#include <dbus/dbus-glib.h>
+#include <dbus/dbus-glib-lowlevel.h>
+
+
 #include "gnome-vfs-hal-mounts.h"
 #include "gnome-vfs-volume-monitor-daemon.h"
 #include "gnome-vfs-volume-monitor-private.h"
@@ -521,9 +531,9 @@ _hal_get_drive_name (GnomeVFSHalVolume *vol, GnomeVFSHalDrive *drive,
 		if( dvdram )
 			second = "/DVD-RAM";
 		if( dvdplusr && dvdr )
-			second = "/DVDR";
+			second = "/DVD\302\261R";
 		if( dvdplusr && dvdplusrw && dvdr )
-			second = "/DVDRW";
+			second = "/DVD\302\261RW";
 
 		name = g_strdup_printf ("%s%s", first, second);
 	} else if (strcmp (drive->type, "floppy") == 0) {
