@@ -621,7 +621,7 @@ do_read (GnomeVFSMethod *method,
 	if (gzip_handle->last_z_result != Z_OK) {
 		if (gzip_handle->last_z_result == Z_STREAM_END) {
 			*bytes_read = 0;
-			return GNOME_VFS_OK;
+			return GNOME_VFS_ERROR_EOF;
 		} else
 			return result_from_z_result (gzip_handle->last_z_result);
 	} else if (gzip_handle->last_vfs_result != GNOME_VFS_OK) {
@@ -733,7 +733,7 @@ do_get_file_info  (GnomeVFSMethod *method,
 			file_info->name[namelen-3] = '\0';
 
 		/* we can't tell the size without uncompressing it */
-		//file_info->valid_fields &= ~GNOME_VFS_FILE_INFO_FIELDS_SIZE;
+		/*file_info->valid_fields &= ~GNOME_VFS_FILE_INFO_FIELDS_SIZE;*/
 
 		/* guess the mime type of the file inside */
 		/* FIXME bugzilla.eazel.com 2791: guess mime based on contents */
